package ru.dbotthepony.mc.otm.container

import net.minecraft.world.Container
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import net.neoforged.neoforge.items.IItemHandler
import net.neoforged.neoforge.items.IItemHandlerModifiable

/**
 * Reinforced [ISlottedContainer] which slots are [IAutomatedContainerSlot]s, which
 * subsequently allow this container to implement [IItemHandler]
 */
interface IAutomatedContainer : ISlottedContainer, IItemHandlerModifiable {
	override fun containerSlot(slot: Int): IAutomatedContainerSlot

	override fun canPlaceItem(slot: Int, itemStack: ItemStack): Boolean {
		return containerSlot(slot).canAutomationPlaceItem(itemStack)
	}

	override fun canTakeItem(container: Container, slot: Int, itemStack: ItemStack): Boolean {
		return containerSlot(slot).canAutomationTakeItem()
	}

	override fun getSlots() = containerSize
	override fun getStackInSlot(slot: Int) = containerSlot(slot).item

	override fun insertItem(slot: Int, stack: ItemStack, simulate: Boolean): ItemStack {
		return containerSlot(slot).insertItem(stack, simulate)
	}

	override fun extractItem(slot: Int, amount: Int, simulate: Boolean): ItemStack {
		return containerSlot(slot).extractItem(amount, simulate)
	}

	override fun getSlotLimit(slot: Int): Int {
		return containerSlot(slot).maxStackSize
	}

	override fun setStackInSlot(slot: Int, stack: ItemStack) {
		setItem(slot, stack)
	}

	override fun isItemValid(slot: Int, stack: ItemStack): Boolean {
		return canPlaceItem(slot, stack)
	}
}
